@@ -1,7 +1,7 @@
 /* DCTC - a Direct Connect text clone for Linux
  * Copyright (C) 2001 Eric Prevoteau
  *
- * dc_com.c: Copyright (C) Eric Prevoteau <www@ac2i.tzo.com>
+ * dc_com.c: Copyright (C) Eric Prevoteau <www@a2pb.gotdns.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+/*
+$Id: dc_com.c,v 1.10 2003/12/28 08:12:38 uid68112 Exp $
+*/
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -150,7 +153,9 @@ GString *get_a_dc_line(int sck)
 		if((retval==-1)||(!FD_ISSET(sck,&inp)))
 		{
 			perror("get_a_dc_line - select");
+#if 0
 			printf("%s\n",s->str);
+#endif
 			g_string_free(s,TRUE);
 			return NULL;
 		}
@@ -158,7 +163,9 @@ GString *get_a_dc_line(int sck)
 		n=read(sck,&c,1);
 		if(n!=1)
 		{
+#if 0
 			printf("%s\n",s->str);
+#endif
 			g_string_free(s,TRUE);
 			return NULL;
 		}
@@ -170,21 +177,64 @@ GString *get_a_dc_line(int sck)
 	return s;
 }
 
-/* ------------------------------------------------------------------------------------------- */
-/* ------------------------------------------------------------------------------------------- */
-/* ------------------------------------------------------------------------------------------- */
-/* ------------------------------------------------------------------------------------------- */
-static void send_str_to_dctc_link(GString *str)
+/* contains pointers on the path (stored in dctc_gsc) of all '.udp' files of other DCTC */
+static GPtrArray *running_dctc=NULL;
+/* this array contains the paths themselves */
+static GStringChunk *dctc_gsc=NULL;
+
+/*****************************************************************/
+/* get the number of running DCTC (without counting this client) */
+/*****************************************************************/
+int num_running_dctc(void)
+{
+	int delta=0;
+	int i;
+
+	if(running_dctc==NULL)
+	{
+		refresh_known_client_list(TRUE);
+		if(running_dctc==NULL)
+			return 1;
+	}
+
+	/* adjust the truly running client byt removing the dummy_client if it exists */
+	for(i=0;i<running_dctc->len;i++)
+	{
+		const char *a;
+		int ln;
+
+		a=g_ptr_array_index(running_dctc,i);
+		ln=strlen(a);
+
+		if(ln>=strlen("-dummy_client.udp"))
+		{
+			if(!strcmp(a+ln-strlen("-dummy_client.udp"),"-dummy_client.udp"))
+			{
+				delta=-1;
+				break;
+			}
+		}
+	}
+
+	return (running_dctc->len) + delta +1;		/* -1 because the dummy_client is counted as a running client but it is never connected to anything */
+														/* but +1 because the list does not contained the current client */
+}
+
+/*********************************************/
+/* refresh the list of known running clients */
+/***************************************************/
+/* if force==TRUE, the refresh is forced, else     */
+/* a delay of at least 60 seconds will be observed */
+/***************************************************/
+void refresh_known_client_list(gboolean force)
 {
-	static GPtrArray *running_dctc=NULL;
-	static GStringChunk *dctc_gsc=NULL;
 	static time_t last_update=0;
-	static GString *running_dir=NULL;
 	time_t cur_time;
+	static GString *running_dir=NULL;
 	DIR *dir;
 
 	cur_time=time(NULL);
-	if((cur_time-last_update)>60)
+	if(((cur_time-last_update)>60)||(force==TRUE))
 	{
 		/* refresh the running dctc list every 60 seconds */
 		last_update=cur_time;
@@ -227,7 +277,9 @@ static void send_str_to_dctc_link(GString *str)
 					continue;
 
 				g_string_sprintf(np,"%s%s",running_dir->str,obj->d_name);
-				if(strcmp(np->str,local_dctc_udp_sock_path->str))
+
+				/* don't have our local udp socket to the list */
+				if((local_dctc_udp_sock_path) && (strcmp(np->str,local_dctc_udp_sock_path->str)))
 					g_ptr_array_add(running_dctc,g_string_chunk_insert(dctc_gsc,np->str));
 			}
 
@@ -235,6 +287,16 @@ static void send_str_to_dctc_link(GString *str)
 			closedir(dir);
 		}
 	}
+}
+
+/* ------------------------------------------------------------------------------------------- */
+/* ------------------------------------------------------------------------------------------- */
+/* ------------------------------------------------------------------------------------------- */
+/* ------------------------------------------------------------------------------------------- */
+static void send_str_to_dctc_link(GString *str)
+{
+
+	refresh_known_client_list(FALSE);
 
 	if((running_dctc!=NULL)&&(running_dctc->len!=0))
 	{
@@ -253,7 +315,7 @@ static void send_str_to_dctc_link(GString *str)
 
 				if(sendto(local_sck_udp,str->str,str->len,MSG_NOSIGNAL|MSG_DONTWAIT,(void*)&name,sizeof(struct sockaddr_un))!=str->len)
 				{
-					disp_msg(ERR_MSG,"send_str_to_dctc_link","sendto fail",strerror(errno),NULL);
+					disp_msg(DEBUG_MSG,"send_str_to_dctc_link","sendto fail",strerror(errno),NULL);
 				}
 			}
 		}
@@ -262,7 +324,7 @@ static void send_str_to_dctc_link(GString *str)
 
 /**********************************************************************/
 /* this function works like send_dc_line except that the built string */
-/* is sent to all other running client (if with_dctclink is set)      */
+/* is sent to all other running client                                */
 /**********************************************************************/
 /* an optionnal condition can be added to the sent command */
 /***********************************************************/
@@ -273,9 +335,6 @@ void send_dc_line_to_dctc_link(char *condition,int sck,...)
 	int have=0;
 	GString *str;
 
-	if(with_dctclink==0)
-		return;
-
 #if 0
 	if(sck==-1)		/* sometimes, after a network error, it is possible to come here with an invalid socket descriptor */
 		return;
@@ -332,7 +391,7 @@ void send_dc_line_to_dctc_link(char *condition,int sck,...)
 
 /********************************************************************************************/
 /* this function works like send_dc_line_to_dctc_link except that it takes the given string */
-/* as is append a \n and send it to all other running clients (if with_dctclink is set)     */
+/* as is append a \n and send it to all other running clients                               */
 /********************************************************************************************/
 /* an optionnal condition can be added to the sent command */
 /***********************************************************/
@@ -340,9 +399,6 @@ void send_dc_line_to_dctc_link_direct(char *condition,char *cmd)
 {
 	GString *str;
 
-	if(with_dctclink==0)
-		return;
-
 	/* prepend the condition if it exists */
 	if(condition==NULL)
 	{
