@@ -1,7 +1,7 @@
 /* DCTC - a Direct Connect text clone for Linux
  * Copyright (C) 2001 Eric Prevoteau
  *
- * dc_com.h: Copyright (C) Eric Prevoteau <www@ac2i.tzo.com>
+ * dc_com.h: Copyright (C) Eric Prevoteau <www@a2pb.gotdns.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+/*
+$Id: dc_com.h,v 1.4 2003/12/28 08:12:38 uid68112 Exp $
+*/
 
 #ifndef __DC_COM_H__
 #define __DC_COM_H__
@@ -55,9 +58,17 @@ void send_dc_line(int sck,...);
 /*******************************************************************************/
 GString *get_a_dc_line(int sck);
 
+/*********************************************/
+/* refresh the list of known running clients */
+/***************************************************/
+/* if force==TRUE, the refresh is forced, else     */
+/* a delay of at least 60 seconds will be observed */
+/***************************************************/
+void refresh_known_client_list(gboolean force);
+
 /**********************************************************************/
 /* this function works like send_dc_line except that the built string */
-/* is sent to all other running client (if with_dctclink is set)      */
+/* is sent to all other running client                                */
 /**********************************************************************/
 /* an optionnal condition can be added to the sent command */
 /***********************************************************/
@@ -65,7 +76,7 @@ void send_dc_line_to_dctc_link(char *condition,int sck,...);
 
 /********************************************************************************************/
 /* this function works like send_dc_line_to_dctc_link except that it takes the given string */
-/* as is append a \n and send it to all other running clients (if with_dctclink is set)     */
+/* as is append a \n and send it to all other running clients                               */
 /********************************************************************************************/
 /* an optionnal condition can be added to the sent command */
 /***********************************************************/
@@ -76,4 +87,9 @@ void send_dc_line_to_dctc_link_direct(char *condition,char *cmd);
 /**************************************/
 void send_str_to_unode(GString *str);
 
+/*****************************************************************/
+/* get the number of running DCTC (without counting this client) */
+/*****************************************************************/
+int num_running_dctc(void);
+
 #endif
