@@ -1,7 +1,7 @@
 /* DCTC - a Direct Connect text clone for Linux
  * Copyright (C) 2001 Eric Prevoteau
  *
- * key.c: Copyright (C) Eric Prevoteau <www@ac2i.tzo.com>
+ * key.c: Copyright (C) Eric Prevoteau <www@a2pb.gotdns.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+/*
+$Id: key.c,v 1.5 2003/12/28 08:12:38 uid68112 Exp $
+*/
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -271,8 +274,6 @@ int verify_key(char *c_lock,GString *input, int lazy_mode)
 
 	if(strncmp(input->str+5,key->str,key->len))
 	{
-		int i;
-
 		if((lazy_mode)&&(key->len>3))
 		{
 			GString *partial_key;
@@ -294,10 +295,16 @@ int verify_key(char *c_lock,GString *input, int lazy_mode)
 		disp_msg(ERR_MSG,"verify_key","wanted:",key->str,NULL);
 		disp_msg(ERR_MSG,"verify_key","have:  ",input->str+5,NULL);
 		
-		for(i=0;i<key->len;i++)
+#if 0
 		{
-			printf("%c(%02X) ",key->str[i],((int)key->str[i])&255);
+			int i;
+
+			for(i=0;i<key->len;i++)
+			{
+				printf("%c(%02X) ",key->str[i],((int)key->str[i])&255);
+			}
 		}
+#endif
 		g_string_free(key,TRUE);
 		return 2;
 	}
