@@ -1,7 +1,7 @@
 /* DCTC - a Direct Connect text clone for Linux
  * Copyright (C) 2001 Eric Prevoteau
  *
- * key.h: Copyright (C) Eric Prevoteau <www@ac2i.tzo.com>
+ * key.h: Copyright (C) Eric Prevoteau <www@a2pb.gotdns.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+/*
+$Id: key.h,v 1.2 2003/12/28 08:12:38 uid68112 Exp $
+*/
 
 #ifndef __KEY_H__
 #define __KEY_H__
