@@ -1,7 +1,7 @@
 /* DCTC - a Direct Connect text clone for Linux
  * Copyright (C) 2001 Eric Prevoteau
  *
- * main.c: Copyright (C) Eric Prevoteau <www@ac2i.tzo.com>
+ * main.c: Copyright (C) Eric Prevoteau <www@a2pb.gotdns.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+/*
+$Id: main.c,v 1.29 2003/12/28 08:12:38 uid68112 Exp $
+*/
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -68,12 +71,14 @@
 #include "status.h"
 #include "bdb.h"
 #include "userinfo.h"
+#include "misc.h"
+#include "md_db.h"
 
 /*****************************************************/
 /* this mutex locks all pointers of user description */
 /*****************************************************/
-/* nickname, user_desc, cnx_type, email, host_ip */
-/*************************************************/
+/* nickname, user_desc, cnx_type, email, org_host_ip */
+/*****************************************************/
 HL_MUTEX user_info=HL_MUTEX_INIT;		/* this is a high level mutex allowing multiple readers or 1 writer */
 
 /*******************************/
@@ -98,14 +103,6 @@ double sizeof_data=0;			 	/* size of the data shared by this client */
 double offset_sizeof_data=0;     /* when the client has to return the size of data it shares, this value */
                                  /* is added to the previous one */
 
-#if 0
-int free_dl_slot;						/* number of free download slot */
-											/* NOTE: this number can be negative for 2 reasons: */
-											/* 1) you have reduced the ttl_dl_slot value under the number of running uploads */
-											/* 2) a user having the IGNORE_SLOT_LIMIT flag is downloading */
-int ttl_dl_slot;						/* number of existing download slot */
-#endif
-
 int dl_on;								/* this flag enables or disables the upload capability */
 											/* if ==1, other users can download from you */
 											/* if ==0, other users cannot download from you */
@@ -118,6 +115,8 @@ char *nick_passwd=NULL;				/* when the hub wants a password. We will use the one
 
 GString *dctc_rating_gs=NULL;		/* this value is not used by DCTC, it is only kept here */
 
+int description_tag_auto=FALSE;	/* if set, the client will automatically attached tag to the user description */
+
 /**************************************/
 /* description of the connection type */
 /**************************************/
@@ -128,8 +127,10 @@ unsigned short com_port;	      /* port available, to receive data if not behind
 											/* default value is 412 but it is not very good     */
 											/* because port below 1024 are not allowed for      */
 											/* non-privileged users under Un*x                  */
-char *host_ip;					      /* ip of this host */
-
+char *host_ip=NULL;			      /* ip of this host */
+char *org_host_ip=NULL;				/* this is where the command /IP put its content */
+											/* because the command can provided a FQDN or even a network interface */
+											/* we cannot load directly host_ip with it */
 int dynamic_ip_flag=0;				/* if ==0, once host_ip has been filled, it is never updated (static IP) */
 											/* else ==1, the host_ip is recomputed each time before connecting the hub */
 
@@ -174,22 +175,9 @@ int keyb_fd=0;					/* it is the keyboard fd */
 int when_done=0;				/* when a download is done, ==0, do nothing */
 									/* ==1, move the file into the done/ directory */
 
-#define MAX_ALLOWED_EMODE 1
-int wanna_emode=0;			/* when the --mdc flag is used, this variable contains the flag value */
-int having_emode=0;			/* depending of the connected hub, the current emode may differ from the */
-									/* wanted emode */
-
-int with_md5sum=1;			/* enable/disable md5sum for content search */
-
 int with_ddl=0;				/* enable/disable dDL capability */
-int with_dctclink=0;			/* enable/disable link between dctc clients */
 int grab_ban_ip=0;			/* enable/disable IP grabbing of banned users */
 
-int with_dl_force=0;			/* enable/disable download forcing when a twin download occurs */
-
-int hide_absolute=1;			/* hide/show the / (\ seen on DC) at the beginning of the returned search result and file list */
-									/* default mode: hide absolute path */
-
 int abort_upload_when_user_leaves=0;
 									/* abort the remote user upload when a user leaves the hub */
 
@@ -211,6 +199,15 @@ int gdl_as_port_range[2]={0,0};		/* lower and upper value of GDL autoscan port r
 
 int min_delay_between_search=0;		/* delay to wait between 2 search queries */
 
+unsigned int sharelist_dl_wo_slot=0;	/* allow downloading of your sharelist by clients even if no slots are available */
+
+unsigned int fake_dcpp_client=0;			/* simulate DC++ tag and key */
+GString *fake_dcpp_version=NULL;			/* version of DC++ to simulate */
+
+unsigned int disp_user=1;					/* disable/enable USER+,USER-, ... and anything related to user change message (1=enable, 0=disable)*/
+
+GPtrArray *client_capabilities=NULL;	/* list of capabilities of the client */
+
 /************************************************************/
 /* debug mode. ==0, no debug. !=0, debug mode               */
 /* if debug_mode is set, all debug messages will be printed */
@@ -361,6 +358,9 @@ void hub_disconnect(HUBDISC_FLAG exit_flag)
 	if(local_dctc_user_info_path!=NULL)
 		unlink(local_dctc_user_info_path->str);
 
+	/* say hello to other local DCTCs */
+	send_dc_line_to_dctc_link_direct(NULL,"/HELLODCTC");
+
 	/* exit BerkeleyDB lib */
 	do_berkeley_exit();
 
@@ -395,60 +395,6 @@ static void	expire_old_xfer(void)
 				/* remove action from the list */
 				g_ptr_array_remove_index(waiting_revcon,i);
 
-#if 0
-				if(!strncmp(ptr->action_to_do->str,"DL/",3))
-				{
-					/* we will requeue this xfer inside sim_input */
-					{
-						GString *sim;
-						GString *local;
-						GString *org_remote;
-						char *t;
-						char sep;
-
-						uaddr_increase_error_flag(ptr->remote_nick->str);
-
-						sep=ptr->action_to_do->str[sizeof("DL/")-1]; /* get the separator */
-
-						local=g_string_new(ptr->action_to_do->str + sizeof("DL/") -1 + 1 /* ignore the separator */ );
-						t=strrchr(local->str,sep);		 /* can't fail */
-						local=g_string_truncate(local,t-local->str);	 /* truncate the size of the string in local */
-
-						t=strrchr(local->str,sep);		 /* can't fail */
-						org_remote=g_string_new(t+1);
-						local=g_string_truncate(local,t-local->str);	 /* truncate the remote path of the string in local */
-
-						sim=g_string_new("");
-
-						/* we use | as separator because it cannot appear anywhere */
-						g_string_sprintf(sim,"/DL |%s|%s|%s|",
-											ptr->remote_nick->str, local->str, org_remote->str);
-						/* @@@ add the xtra information at the end of the /DL */
-						{
-							UCNX tp;
-							GString *xt;
-
-							xt=get_xtra_information_cnx(ptr->remote_nick->str,&tp);
-							if((tp==ACTIVE)&&(xt!=NULL)&&(xt->len!=0))
-							{
-								g_string_sprintfa(sim,"|%s",xt->str);
-							}
-							if(xt!=NULL)
-								g_string_free(xt,TRUE);
-						}
-
-						/* wait 30 seconds before retrying */
-						if(add_gts_entry(ptr->remote_nick->str,sim->str,60))
-            			add_new_sim_input(60,sim->str);     /* fail to queue in the GTS, use sim_input instead */
-
-						g_string_free(sim,TRUE);
-						g_string_free(local,TRUE);
-						g_string_free(org_remote,TRUE);
-					}
-					disp_msg(XFER_DL_UNQUEUED,NULL, ptr->remote_nick->str,ptr->action_to_do->str,NULL);
-				}
-				else 
-#endif
 				if(!strncmp(ptr->action_to_do->str,"LS/",3))
 				{
 					uaddr_increase_error_flag(ptr->remote_nick->str);
@@ -476,8 +422,14 @@ static void	expire_old_xfer(void)
 
 					uaddr_increase_error_flag(ptr->remote_nick->str);
 
-					/* notify to GDL the transfer never start */
-					fields=g_strsplit(ptr->action_to_do->str,"|",3);		/* there is always 3 fields, never less, never more */
+					/* notify to GDL the transfer never starts */
+					/* there is always 3 fields, never less, never more */
+#ifndef WITH_GLIB2
+					fields=g_strsplit(ptr->action_to_do->str,"|",3);     /* GLIB2 fixed */
+#else
+					fields=g_strsplit(ptr->action_to_do->str,"|",3+1);   /* GLIB2 fix */
+#endif
+
 					do_gdl_abort(strtoul(fields[1],NULL,10),fields[2]);
 					g_strfreev(fields);
 					disp_msg(DEBUG_MSG,NULL, "expiring XDL","|lu",(unsigned long)ptr,NULL);
@@ -680,9 +632,6 @@ static void	manage_local_udp_socket(int sck)
 		return;
 	}
 
-	if(with_dctclink==0)
-		return;
-
 	buf[ret]='\0';
 	printf("unix_udp_socket: %s\n",buf);
 
@@ -725,23 +674,13 @@ static void	do_login_stage(void)
 	}
 
 	cnx_in_progress=0;			/* socket is connected */
+	wipe_GPtrArray_of_char_string(&hub_capabilities);
 	display_cnx_status();
 	reset_hub_user_list();
 
 	/* before all, we must pass the hub challenge */
 	if((i=unlock_access(main_sck,NULL))!=0)
 	{
-#if 0
-		GString *cur_input;
-
-		cur_input=get_a_dc_line(main_sck);
-		while(cur_input!=NULL)
-		{
-			disp_msg(INFO_MSG,"",cur_input->str,NULL);
-			g_string_free(cur_input,TRUE);
-			cur_input=get_a_dc_line(main_sck);
-		}
-#else
 		GString *cur_input;
 
 		cur_input=get_a_dc_line(main_sck);
@@ -753,7 +692,6 @@ static void	do_login_stage(void)
 			cur_input=get_a_dc_line(main_sck);
 		}
 
-#endif
 		disp_msg(HUB_DISCONNECT,"","Hub has closed its connection.",NULL);
 		hub_disconnect(DO_RECONNECT);
 		return;
@@ -764,11 +702,6 @@ static void	do_login_stage(void)
 	/* do the login stage */
 	LOCK_READ(user_info);
 	send_dc_line(main_sck,"$ValidateNick",nickname,NULL);		/* send our nickname */
-#if 0
-	send_dc_line(main_sck,"$Version",dc_version,NULL);					/* nothing is returned */
-	send_dc_line(main_sck,"$GetNickList",NULL);					/* get list of all nickname */
-	set_user_info(main_sck,nickname,user_desc,cnx_type,cnx_opt,email,sizeof_data);
-#endif
 	UNLOCK_READ(user_info);
 }
 
@@ -802,9 +735,13 @@ static void main_loop()
 	fd_set rd,wd;
 	struct timeval tv;
 	int i;
+	time_t last_socket_event=time(NULL);
+	time_t cur_time;
 
 	while(1)
 	{
+		cur_time=time(NULL);
+
 		n=-1;
 
 		FD_ZERO(&rd);
@@ -882,6 +819,7 @@ static void main_loop()
 			else if( (main_sck!=-1) && (cnx_in_progress==0) && (FD_ISSET(main_sck,&rd)) )
 			{	/* something from the hub ? */
 				get_dc_line_and_process(main_sck);
+				last_socket_event=cur_time;
 			}
 			else if((in_sck!=-1)&&(FD_ISSET(in_sck,&rd)))
 			{	/* someone wants to connect to this client ? */
@@ -950,7 +888,7 @@ static void main_loop()
 
 		if(user_wants_to_quit)
 		{
-			disp_msg(ERR_MSG,NULL,"use /QUIT to quit",NULL);
+			disp_msg(DEBUG_MSG,NULL,"use /QUIT to quit",NULL);
 			user_wants_to_quit=0;
 		}
 
@@ -968,8 +906,20 @@ static void main_loop()
 
 			LOCK_READ(user_info);
 			set_user_info(main_sck,nickname, user_desc, cnx_type,cnx_opt,email,sizeof_data);
+			last_socket_event=cur_time;
 		}
 		UNLOCK_READ(user_info);
+
+		if((cur_time-last_socket_event)>(2*60-1))	/* max delay without socket activity */
+		{
+			/* if we are connected to the hub */
+			if ((main_sck!=-1) && (cnx_in_progress==0))
+			{
+				send_dc_line(main_sck,"",NULL);		/* send a "|" on the socket to keep the socket alive, as nmdc */
+			}
+
+			last_socket_event=cur_time;
+		}
 	}
 }
 
@@ -997,12 +947,14 @@ static void set_sig(void)
 	sigemptyset(&set);
 	sigaddset(&set,SIGPIPE);
 	sigaddset(&set,SIGHUP);
+	sigaddset(&set,SIGCHLD);		/* we want to discard potential children */
 	act.sa_handler=SIG_IGN;
 	act.sa_mask=set;
 	act.sa_flags=SA_RESTART;
 
 	sigaction(SIGPIPE,&act,NULL);
 	sigaction(SIGHUP,&act,NULL);
+	sigaction(SIGCHLD,&act,NULL);
 	sigprocmask(SIG_UNBLOCK,&set,NULL);
 
 	/* and catch some other sig */
@@ -1051,6 +1003,8 @@ static void display_usage(char *fname)
 						"  -t, --no_tty                    Detach from the tty used by the shell\n"
 						"                                  starting the program\n"
 						"  -l, --link                      Wait a connection on the unix socket before\n"
+						"  -m, --mark                      Add DCTC tag to user description\n"
+						"  -Z, --socket_dir=DIR            Change the unix socket path (default: ~/.dctc)\n"
 						"                                  starting (only useful with a GUI)\n"
 						"  -v, --version=VERSION           Force DC version. Modifying this number\n"
 						"                                  doesn't affect DCTC but it allows you to enter\n"
@@ -1068,9 +1022,6 @@ static void display_usage(char *fname)
 						"                                  if this client creates the bandwidth limit\n"
 						"                                  semaphore, use /[UDG]BL to change the value\n"
 						"                                  later\n"
-						"  -m, --mdc=NUM                   enable non DC hub compatible command\n"
-						"                                  this flag is for future usage. Current default\n"
-						"                                  value is 0.\n"
 						"  -b, --precmd=COMMAND            perform the given command BEFORE connecting to\n"
 						"                                  the hub. Not all / commands can be used here,\n"
 						"                                  look at the command list to know which are.\n"
@@ -1078,23 +1029,20 @@ static void display_usage(char *fname)
 						"                                  --precmd for each one. When multiple --precmd\n"
 						"                                  are given, commands are performed in the given\n"
 						"                                  order.\n"
-						"  -5, --nomd5sum                  Do not compute MD5 sum for each shared file\n"
-						"                                  This sum is used for content search.\n"
-						"                                  Default: enabled\n"
-						"  -k, --dctclink                  some commands can be received from or send to\n"
-						"                                  other running dctc clients. This is useful to\n"
-						"                                  improve search and download efficiency.\n"
+						"  -C, --connectproxy=ADDR[:PORT]  address (IP or name) and port of a web proxy\n"
+						"                                  supporting the CONNECT command. The default\n"
+						"                                  port is 8080 if none is provided.\n"
 						"  -S, --socksaddr=ADDR            address (IP or name) of the SOCKS proxy to use\n"
                   "                                  Default: no SOCKS proxy is used\n"
                   "  -P, --socksport=NUM             port of the SOCKS proxy. Default: 1080\n"
                   "                                  Only used if -S used\n"
-                  "  -U, --socksuid=STRING           If the SOCKS proxy requires a userID, this is\n"
-						"                                  it. Default: none\n"
-						"  -X, --socksv5                   Enable SOCKS v5 support. Default: if -S is used\n"
-						"                                  SOCKS protocol version 4 is used.\n"
-						"  -K, --sockspass=STRING          If the SOCKS proxy requires a password, use this\n"
-						"                                  flag. Note: if this flag is used, you MUST also\n"
-						"                                  use -U to give a userID.\n"
+                  "  -U, --socksuid=STRING           If the SOCKS/Web proxy requires a userID,\n"
+						"                                  this is it. Default: none\n"
+						"  -X, --socksv5                   Enable SOCKS v5 support. Default: if -S is\n"
+						"                                  used, SOCKS protocol version 4 is used.\n"
+						"  -K, --sockspass=STRING          If the SOCKS/Web proxy requires a password,\n"
+						"                                  use this flag. Note: if this flag is used,\n"
+						"                                  you MUST also use -U to give a userID.\n"
 						"  -D, --virtual_share_dir=DIR     Define a directory contains files describing\n"
 						"                                  virtually shared files. (see Docs/Vshare).\n"
 						"\n"
@@ -1212,8 +1160,8 @@ static void *periodic_rebuild_db(void *nothing)
 
 static void start_rebuild_db_thread(void)
 {
-	static pthread_t thread_id;			/* these 2 variables must exist as long as the thread exist */
-	static pthread_attr_t thread_attr;
+	static pthread_t thread_id;			/* this variable must exist as long as the thread exist */
+	pthread_attr_t thread_attr;
 
 	pthread_attr_init (&thread_attr);
 	pthread_attr_setdetachstate(&thread_attr, PTHREAD_CREATE_DETACHED);
@@ -1221,6 +1169,7 @@ static void start_rebuild_db_thread(void)
 	{
 		disp_msg(ERR_MSG,"start_rebuild_db_thread","pthread_create failed","Periodic rebuilding of shared file database disabled",NULL);
 	}
+	pthread_attr_destroy(&thread_attr);
 }
 
 /* beginning of the code */
@@ -1244,21 +1193,21 @@ int main(int argc,char **argv)
 												{"when_done",no_argument,NULL,'w'},			/* move file when download over */
 												{"no_tty",no_argument,NULL,'t'},				/* detach from start shell tty */
 												{"link",no_argument,NULL,'l'},				/* wait someone connection before starting */
+												{"socket_dir",required_argument,NULL,'Z'},/* set the unix socket path */
 												{"version",required_argument,NULL,'v'},	/* force change of version number */
 												{"upload",required_argument,NULL,'u'},		/* upload bandwidth limit */
-												{"mdc",required_argument,NULL,'m'},			/* enable non compatible DC command */
 												{"precmd",required_argument,NULL,'b'},		/* send a command to execute before hub connection */
-												{"nomd5sum",no_argument,NULL,'5'},			/* don't compute md5 sum for each file of the database */
-												{"dctclink",no_argument,NULL,'k'},			/* enable inter dctc communication */
+												{"connectproxy",required_argument,NULL,'C'},	/* set the address and port of a web proxy supporting the CONNECT command to use */
 												{"socksaddr",required_argument,NULL,'S'},	/* set the address of the socks proxy to use */
 												{"socksport",required_argument,NULL,'P'},	/* set the port of the socks proxy to use */
 												{"socksuid",required_argument,NULL,'U'},	/* set the userID to use with the socks proxy */
 												{"sockspass",required_argument,NULL,'K'},	/* set the user password to use with the socks proxy */
 												{"socksv5",no_argument,NULL,'X'},			/* use SOCKS v5 instead of SOCKS v4 */
+												{"mark",no_argument,NULL,'m'},				/* automatically add a mark (tag) to the user description */
 												{"virtual_share_dir",required_argument,NULL,'D'},	/* directory containing files containing virtual shared files */
 												{NULL,0,NULL,'\0'}					/* last option */
 											};
-	static const char *short_opt="hn:i:c:e:d:s:o:a:p:g:fT:xwtlv:u:m:b:5kS:P:U:D:";
+	static const char *short_opt="hn:i:c:e:d:s:o:a:p:g:fT:xwtlv:u:b:5kS:P:U:D:Z:mC:XK:";
 										
 	int ch;
 	int detach_from_tty=0;
@@ -1268,6 +1217,8 @@ int main(int argc,char **argv)
 	int gath_spd_limit=SEMVMX;
 	int ttl_dl_slot;
 
+	char *socket_dir;
+
 	/* the 2 sh_* are used to stored -s directories */
 	GStringChunk *sh_gsc;
 	GPtrArray *sh_gpa;
@@ -1301,9 +1252,10 @@ int main(int argc,char **argv)
 
 	/* to be able to accept connection, we must set few values */
 	behind_fw=0;
-	host_ip=NULL;
 	com_port=412;
 
+	socket_dir=NULL;
+
 	sh_gsc=g_string_chunk_new(128);
 	sh_gpa=g_ptr_array_new();
 
@@ -1346,16 +1298,9 @@ int main(int argc,char **argv)
 								ttl_dl_slot=0;
 							break;
 
-			case 'm':	wanna_emode=atoi(optarg);
-							if(wanna_emode<0)
-								wanna_emode=0;
-							if(wanna_emode>MAX_ALLOWED_EMODE)
-								wanna_emode=0;
-							break;
-
-			case 'a':	if(host_ip!=NULL)
-								free(host_ip);
-							host_ip=strdup(optarg);
+			case 'a':	if(org_host_ip!=NULL)
+								free(org_host_ip);
+							org_host_ip=strdup(optarg);
 							break;
 
 			case 'p':	com_port=strtoul(optarg,NULL,10);
@@ -1451,6 +1396,9 @@ int main(int argc,char **argv)
 			case 'l':	wait_lnk=1;
 							break;
 
+			case 'm':	description_tag_auto=TRUE;
+							break;
+
 			case 'v':	dc_version=strdup(optarg);
 							break;
 
@@ -1466,10 +1414,13 @@ int main(int argc,char **argv)
 			case 'b':	g_ptr_array_add(pre_gpa,g_string_chunk_insert(pre_gsc,optarg));
 							break;
 
-			case '5':	with_md5sum=0;
-							break;
-
-			case 'k':	with_dctclink=1;
+			case 'C':	/* web proxy address */
+							if(web_proxy_address!=NULL)
+								free(web_proxy_address);
+							if(strlen(optarg)==0)
+								web_proxy_address=NULL;
+							else
+								web_proxy_address=strdup(optarg);
 							break;
 
 			case 'S':	/* SOCKS address */
@@ -1509,9 +1460,13 @@ int main(int argc,char **argv)
 			case 'D':	virtual_share_path=strdup(optarg);
 							break;
 
+			case 'Z':	/* unix socket path */
+							socket_dir=strdup(optarg);
+							break;
+
 			default:
-							fprintf(stderr,"Unknown option: %c\n",ch);
-							exit(1);
+							fprintf(stderr,"Unknown option: %c (ignored)\n",ch);
+							break;
 		}
 	}
 
@@ -1533,7 +1488,7 @@ int main(int argc,char **argv)
 	{
 		GString *hip;
 
-		hip=get_default_host_ip();
+		hip=get_default_host_ip(org_host_ip,dynamic_ip_flag);
 		if(hip==NULL)
 		{
 			fprintf(stderr,"Unable to obtain localhost IP.\n");
@@ -1558,7 +1513,7 @@ int main(int argc,char **argv)
 
 		a=open("/dev/null",O_WRONLY);
 		if(a==-1)
-			exit(fprintf(stderr,"unable to open /dev/null"));
+			exit(fprintf(stderr,"unable to open /dev/null\n"));
 
 		dup2(a,0);
 		dup2(a,1);
@@ -1576,24 +1531,22 @@ int main(int argc,char **argv)
 	/* run precmd commands */
 	{
 		int i;
+		gboolean has_capab=FALSE;
 
 		for(i=0;i<pre_gpa->len;i++)
 		{
+			if(!strncmp(g_ptr_array_index(pre_gpa,i),"/CAPAB ",strlen("/CAPAB ")))
+				has_capab=TRUE;
 			process_precmd_command(g_ptr_array_index(pre_gpa,i));
 		}
 		g_ptr_array_free(pre_gpa,TRUE);
 		g_string_chunk_free(pre_gsc);
-	}
 
-	/* hub communication socket */
-	hub_logged=0;
-	if(wanna_emode)
-	{
-		/* if the emode is enabled, first try to open the emode socket */
-		main_sck=create_and_open_sock_on(hubip->str,hub_port+1,0);
-		if(main_sck>=0)
-			goto emode_connected;
-		having_emode=wanna_emode;
+		if(has_capab==FALSE)
+		{
+			disp_msg(ERR_MSG,"","No client capabilities set: default to '/CAPAB MD4x'.",NULL);
+			process_precmd_command("/CAPAB MD4x");
+		}
 	}
 
 	if(strcmp(hubip->str,"dummy_client"))
@@ -1607,7 +1560,6 @@ int main(int argc,char **argv)
 		}
 	}
 
-	emode_connected:
 	set_tos_sock(main_sck,hub_tos);
 	cnx_in_progress=1;
 
@@ -1644,9 +1596,15 @@ int main(int argc,char **argv)
 		char *path;
 		struct stat st;
 
-		path=getenv("HOME");
 		local_dctc_sock_path=g_string_new(NULL);
-		g_string_sprintf(local_dctc_sock_path,"%s/.dctc",(path!=NULL)?path:".");
+
+		if (socket_dir) {
+			g_string_sprintf(local_dctc_sock_path,"%s",socket_dir);
+			free(socket_dir);
+		} else {
+			path=getenv("HOME");
+			g_string_sprintf(local_dctc_sock_path,"%s/.dctc",(path!=NULL)?path:".");
+		}
 
 		if(stat(local_dctc_sock_path->str,&st))
 		{
@@ -1685,6 +1643,16 @@ int main(int argc,char **argv)
 			}
 		}
 
+		/* initialize the md database */
+		{
+			gchar *dctc_md_db=g_strconcat(dctc_dir->str,"/md_db",NULL);
+			if(init_md_db(dctc_md_db))
+			{
+				disp_msg(ERR_MSG,"","fail to create MD database file (",dctc_md_db,").",NULL);
+				exit(1);
+			}
+			g_free(dctc_md_db);
+		}
 
 		g_string_sprintfa(local_dctc_sock_path,"/dctc-%08X-%s",getpid(),org_hubip->str);
 
@@ -1750,6 +1718,7 @@ int main(int argc,char **argv)
 
 	init_gts();
 	init_uaddr();
+	reset_hub_user_list();
 
 	/* perform bandwidth limitation semaphore init */
 	{
@@ -1829,6 +1798,9 @@ int main(int argc,char **argv)
 
 	start_gdl_thread();
 
+	/* say hello to other local DCTCs */
+	send_dc_line_to_dctc_link_direct(NULL,"/HELLODCTC");
+
 	main_loop();
 
 	/* despite of what it looks like, everything below this comment is never used */
@@ -1842,6 +1814,12 @@ int main(int argc,char **argv)
 	close(main_sck);
 	main_sck=-1;
 
+	/* remove the .udp socket entry and force an update of other client before closing our UDP socket */
+	unlink(local_dctc_udp_sock_path->str);
+
+	/* say hello to other local DCTCs */
+	send_dc_line_to_dctc_link_direct(NULL,"/HELLODCTC");
+
 	/* close unix sockets */
 	close(local_sck);
 	local_sck=-1;
@@ -1861,7 +1839,6 @@ int main(int argc,char **argv)
 	G_UNLOCK(local_client_socket);
 	
 	unlink(local_dctc_sock_path->str);
-	unlink(local_dctc_udp_sock_path->str);
 	uinfo_delete();
 	unlink(local_dctc_user_info_path->str);
 
