@@ -1,7 +1,7 @@
 /* DCTC - a Direct Connect text clone for Linux
  * Copyright (C) 2001 Eric Prevoteau
  *
- * db.c: Copyright (C) Eric Prevoteau <www@ac2i.tzo.com>
+ * db.c: Copyright (C) Eric Prevoteau <www@a2pb.gotdns.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+/*
+$Id: mydb.c,v 1.14 2004/01/03 16:22:16 ericprev Exp $
+*/
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -44,6 +47,8 @@
 #include "md.h"
 #include "macro.h"
 #include "sema.h"
+#include "md_crc.h"
+#include "md_db.h"
 
 /* directory containing virtual shares */
 char *vshare_directory=NULL;
@@ -88,6 +93,7 @@ static int get_file_type_from_ext(char *fullname)
 						{"ogg",2},
 						{"au",2},
 						{"wav",2},
+						{"mid",2},		/* midi files */
 
 						{"8svx",2},	/* the following file extension comes from SOX manual */
 						{"aiff",2},
@@ -123,6 +129,7 @@ static int get_file_type_from_ext(char *fullname)
 						/* document type */
 						{"doc",4},
 						{"txt",4},
+						{"sub",4},
 						{"nfo",4},
 						{"c",4},
 						{"C",4},
@@ -139,6 +146,10 @@ static int get_file_type_from_ext(char *fullname)
 						/* exe type */
 						{"exe",5},
 						{"com",5},
+						{"iso",5},
+						{"bin",5},
+						{"cue",5},
+						{"img",5},
 
 						/* picture type */
 						{"gif",6},
@@ -166,9 +177,11 @@ static int get_file_type_from_ext(char *fullname)
 						{"m1v",7},
 						{"mpeg",7},
 						{"avi",7},
+						{"ogm",7},
 						{"divx",7},
 						{"mov",7},
 						{"rm",7},
+						{"mkv",7},
 						{"pva",7},	/* DVB card video format */
 
 						{NULL,-1}		/* last line */
@@ -180,11 +193,7 @@ static int get_file_type_from_ext(char *fullname)
 	/* find filename */
 	fname=strrchr(fullname,'/');
 	if(fname==NULL)
-#if 0
-		return -1;						/* bad name */
-#else
 		fname=fullname;
-#endif
 
 	/* find extension */
 	ext=strrchr(fname,'.');
@@ -251,42 +260,13 @@ static GArray *add_db_entry(GArray *db, char *fullname, double *sod,int *is_dir,
 	/* if file is big enough, we compute its MD5SUM */
 	if(de.filesize>BLOCKSIZE)
 	{
-		if(with_md5sum)
-		{
-			FILE *f;
-			char fcontent[BLOCKSIZE];
-
-			f=fopen(fullname,"rb");
-			if(f==NULL)
-			{
-				disp_msg(ERR_MSG,"add_db_entry","unable to open file",fullname,NULL);
-				return db;
-			}
-
-			if(fread(fcontent,1,BLOCKSIZE,f)!=BLOCKSIZE)
-			{
-				fclose(f);
-				disp_msg(ERR_MSG,"add_db_entry","unable to read enough data from",fullname,NULL);
-				return db;
-			}
-		
-			fclose(f);
-
-			md5_process_block(fcontent,de.md5sum);
-		}
-		else
-		{
-			/* no md5sum support -> clear the checksum */
-			memset(de.md5sum,0,MD5SUMLEN);
-		}
+		/* no md5sum support -> clear the checksum */
+		memset(de.md5sum,0,MD5SUMLEN);
 	}
 
 	if(fullname[0]=='/')
 	{
-		if(hide_absolute==0)
-			de.filename=strdup(fullname);
-		else
-			de.filename=strdup(fullname+1);
+		de.filename=strdup(fullname+1);
 	}
 	else
 		de.filename=strdup(fullname);
@@ -327,10 +307,7 @@ static void add_initial_dir_to_ls(GString **ls_format,int *level, const char *pa
 	if(*cpy=='/')
 	{
 		*cpy=255;
-		if(hide_absolute==0)
-			t=cpy;
-		else
-			t=cpy+1;
+		t=cpy+1;
 	}
 	else
 		t=cpy;
@@ -381,12 +358,7 @@ static GArray *add_one_vshare_entry_to_ls(GArray *db,double *sod,GString **ls_fo
 	memset(de.md5sum,0,MD5SUMLEN);
 
 	if(fullname[0]=='/')
-	{
-		if(hide_absolute==0)
-			de.filename=strdup(fullname);
-		else
-			de.filename=strdup(fullname+1);
-	}
+		de.filename=strdup(fullname+1);
 	else
 		de.filename=strdup(fullname);
 
@@ -635,7 +607,11 @@ static GArray *rebuild_dir_database(GArray *db,double *sod,char *path, GString *
 		else if(fsize!=0)
 		{
 			add_entry_to_ls(ls_format,level,obj->d_name);
+#ifndef __USE_FILE_OFFSET64
 			g_string_sprintfa((*ls_format),"|%lu\r\n",fsize);
+#else
+			g_string_sprintfa((*ls_format),"|%llu\r\n",fsize);
+#endif
 		}
 	}
 	g_string_free(np,TRUE);
@@ -683,9 +659,6 @@ void rebuild_database(void)
 	G_UNLOCK(shared_dirs);
 
 	/* compress the file list using he3 encoding */
-#if 0
-	printf("%s\n",ls_format->str);
-#endif
 	nw_he3=encode_he3_data(ls_format);
 	g_string_free(ls_format,TRUE);
 
@@ -961,16 +934,8 @@ static int pattern_match(GPtrArray *decpat, char *filename)
 
 	for(i=0;i<decpat->len;i++)
 	{
-#if 1
 		if(my_strcasestr(g_ptr_array_index(decpat,i),filename)==NULL)
 			return 0;	/* fail */
-#else
-		char *t;
-		t=my_strcasestr(g_ptr_array_index(decpat,i),filename);
-		printf("%s %s -> %s\n",((GString*)g_ptr_array_index(decpat,i))->str,filename,t);
-		if(t==NULL)
-			return 0;
-#endif
 	}
 	return 1;		/* ok */
 }
@@ -988,11 +953,6 @@ static GString *convert_path(GString *adapted)
 			t[i]='\\';
 	}
 
-#if 0
-	if(adapted->str[0]=='\\')
-		adapted=g_string_erase(adapted,0,1);	/* remove the first characters if it is a / */
-#endif
-	
 	return adapted;
 }
 
@@ -1049,10 +1009,6 @@ static void send_a_db_result(int output_sck, char *dest_nick, DB_ENTRY *de, stru
 	}
 	str=g_string_append_c(str,'|');
 
-#if 0
-	printf("dest_addr: %s, str: %s\n",dest_addr,str->str);
-#endif
-
 	if(dest_addr==NULL)
 		out=send(output_sck,str->str,str->len,MSG_NOSIGNAL);
 	else
@@ -1165,4 +1121,137 @@ int file_in_db(char *filename, int *virtual)
 	return 0;
 }
 
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+/* ------------------------- search in DB by CRC ---------------------------- */
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+/*******************************************************************************************/
+/* search a file having given md4 CRC (MD4_DIGEST_LENGTH bytes) with the given file_length */
+/* in the MD DB and if not found, search inside the shared file db                         */
+/*******************************************************************************************/
+void find_file_with_crc_and_length_and_reply(int output_sck, const char *searching_nick, const guint8 *md4, guint32 file_length)
+{
+	MD_INFO *mi;
+	int i;
+	GString *str;
+	GString *adapted;
+	int out;
+
+	mi=get_md4(md4,file_length);
+	if(mi!=NULL)
+	{
+		str=g_string_new("$MD4Set ");
+		g_string_append(str,searching_nick);
+		g_string_append_c(str,' ');
+		append_MD4_to_str(str,mi->global_crc);
+		g_string_sprintfa(str," %lu ",(unsigned long)mi->filesize);
+		for(i=0;i<mi->nb_seg;i++)
+		{
+			append_MD4_to_str(str,mi->partial_crc+i*MD4_DIGEST_LENGTH);
+		}
+		g_string_append_c(str,' ');
+
+		adapted=g_string_new(mi->filename);
+		adapted=convert_path(adapted);
+		g_string_append(str,adapted->str);
+		g_string_free(adapted,TRUE);
+
+		g_string_append_c(str,'|');
+		
+		out=send(output_sck,str->str,str->len,MSG_NOSIGNAL);
+		if(out==-1)
+		{
+			out=errno;
+			disp_msg(ERR_MSG,"find_file_with_crc_and_length_and_reply","send error:","|d",(int)out,NULL);
+		}
+	
+		g_string_free(str,TRUE);
+		free_md_info(mi);
+		return;
+	}
+
+	/* well, the CRC:filesize is not in the database       */
+	/* let's try to find a file with the matching filesize */
+	G_LOCK(shared_info);
+	if(shared_info!=NULL)
+	{
+		for(i=0;i<shared_info->len;i++)
+		{
+			DB_ENTRY *de;
+	
+			/* DB_ENTRY to compare */
+			de=&(g_array_index(shared_info,DB_ENTRY,i));
+
+			if(de->filesize==file_length)
+			{
+				/* we have a file with the same size, perhaps it is already inside the database */
+				if(!md4_is_inside(file_length,de->filename))
+				{
+					/* no, let's compute the CRC of this file */
+					guint8 comp_g_crc[MD4_DIGEST_LENGTH];
+					guint8 *comp_l_crc;
+					size_t comp_length;
+					guint32 nb_seg;
+					int ret_comp;
+					
+					{
+						GString *full_path;
+
+						full_path=g_string_new(de->filename);
+
+						/* we always prepend a '/' at the beginning of the fullpathname */
+						/* this is done in a very dirty way. Due to the fact that the command is always before the fullname */
+						/* and we won't use it anymore, we "accidentally" trash it :) */
+						g_string_prepend_c(full_path,'/');
+
+						ret_comp=compute_file_crcs(full_path->str,comp_g_crc,&comp_l_crc,&comp_length);
+						g_string_free(full_path,TRUE);
+					}
+
+					if(ret_comp==0)
+					{
+						nb_seg=(comp_length+PARTSIZE-1)/PARTSIZE;
+						md4_add(comp_g_crc,comp_length,comp_l_crc,nb_seg,de->filename);
+
+						if(!memcmp(comp_g_crc,md4,MD4_DIGEST_LENGTH))
+						{
+							str=g_string_new("$MD4Set ");
+							g_string_append(str,searching_nick);
+							g_string_append_c(str,' ');
+							append_MD4_to_str(str,comp_g_crc);
+							g_string_sprintfa(str," %lu ",(unsigned long)comp_length);
+							for(i=0;i<nb_seg;i++)
+							{
+								append_MD4_to_str(str,comp_l_crc+i*MD4_DIGEST_LENGTH);
+							}
+							g_string_append_c(str,' ');
+
+							adapted=g_string_new(de->filename);
+							adapted=convert_path(adapted);
+							g_string_append(str,adapted->str);
+							g_string_free(adapted,TRUE);
+					
+							g_string_append_c(str,'|');
+		
+							out=send(output_sck,str->str,str->len,MSG_NOSIGNAL);
+							if(out==-1)
+							{
+								out=errno;
+								disp_msg(ERR_MSG,"find_file_with_crc_and_length_and_reply","send error:","|d",(int)out,NULL);
+							}
+						
+							g_string_free(str,TRUE);
+							free(comp_l_crc);
+							G_UNLOCK(shared_info);
+							return;
+						}
+						free(comp_l_crc);
+					}
+				}
+			}
+		}
+	}
+	G_UNLOCK(shared_info);
+}
 
