@@ -1,7 +1,7 @@
 /* DCTC - a Direct Connect text clone for Linux
  * Copyright (C) 2001 Eric Prevoteau
  *
- * network.c: Copyright (C) Eric Prevoteau <www@ac2i.tzo.com>
+ * network.c: Copyright (C) Eric Prevoteau <www@a2pb.gotdns.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+/*
+$Id: network.c,v 1.11 2003/12/28 08:12:38 uid68112 Exp $
+*/
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -39,6 +42,7 @@
 #include <errno.h>
 #include <glib.h>
 
+#include "network.h"
 static struct sockaddr_in INET_ZERO = { AF_INET };
 
 /****************************************************************/
@@ -51,6 +55,50 @@ char *socks_pass=NULL;					/* password to use on the socks proxy (or NULL if non
 												/* only valid when socks_name!=NULL AND SOCKSv5 enabled */
 int use_socks_v5=FALSE;					/* prefer SOCKSv5 instead of v4 */
 
+char *web_proxy_address=NULL;			/* address (IP or name) and port of a web proxy to use which supports the command CONNECT */
+
+/* --------------------------------------------------------------------------------------------- */
+static char six2pr[64] = {
+    'A','B','C','D','E','F','G','H','I','J','K','L','M',
+    'N','O','P','Q','R','S','T','U','V','W','X','Y','Z',
+    'a','b','c','d','e','f','g','h','i','j','k','l','m',
+    'n','o','p','q','r','s','t','u','v','w','x','y','z',
+    '0','1','2','3','4','5','6','7','8','9','+','/'
+};
+
+/*********************************************************************************************/
+/* simple function to encode the login:passwd string for proxy requiring an authentification */
+/*********************************************************************************************/
+int uu_auth(unsigned char *bufin, unsigned int nbytes, char *bufcoded)
+{
+#define ENC(c) six2pr[c]
+
+   register char *outptr = bufcoded;
+   unsigned int i;
+   for (i=0; i<nbytes; i += 3) {
+      *(outptr++) = ENC(*bufin >> 2);           /* c1 */
+      *(outptr++) = ENC(((*bufin << 4) & 060) | ((bufin[1] >> 4) & 017)); /*c2*/
+      *(outptr++) = ENC(((bufin[1] << 2) & 074) | ((bufin[2] >> 6) & 03));/*c3*/
+      *(outptr++) = ENC(bufin[2] & 077);        /* c4 */
+
+      bufin += 3;
+   }
+
+   if(i == nbytes+1) {
+      /* There were only 2 bytes in that last group */
+      outptr[-1] = '=';
+   } else if(i == nbytes+2) {
+      /* There was only 1 byte in that last group */
+      outptr[-1] = '=';
+      outptr[-2] = '=';
+   }
+   *outptr = '\0';
+   return(outptr - bufcoded);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
+
 /****************************************************************************************/
 /* perform the socket connection. If socks_ip is NULL, a direct connection is performed */
 /* if socks_ip!=NULL, the function tries to establish a connection thru a SOCKS proxy   */
@@ -64,8 +112,137 @@ static int do_connect_opt_socks(int desc,struct sockaddr_in nom)
 	GByteArray *cmd=NULL;
 
 	if(socks_ip==NULL)
-	{	/* SOCKS proxy disabled */
-		ret=connect(desc,(void*)&nom,sizeof(nom));
+	{	/* SOCKS proxy disabled, web proxy or normal mode can be enabled */
+		if(web_proxy_address==NULL)
+		{	/* normal straight connection mode */
+			ret=connect(desc,(void*)&nom,sizeof(nom));
+		}
+		else
+		{	/* web proxy mode */
+			char buf[512];
+			char *t;
+			struct sockaddr_in web_cnx;
+			unsigned int port;
+			struct hostent *web_host;
+
+			strcpy(buf,web_proxy_address);
+			t=strchr(buf,':');
+			if(t==NULL)
+				port=8080;
+			else
+			{
+				*t++='\0';
+				port=strtoul(t,NULL,10);
+			}
+
+			printf("connecting thru web proxy.\n");
+
+			if((web_host=gethostbyname(buf))==NULL)
+				goto eofunc;
+
+			memcpy(&web_cnx.sin_addr,web_host->h_addr,web_host->h_length);
+			web_cnx.sin_family=AF_INET;
+			web_cnx.sin_port=htons(port);
+
+			ret=connect(desc,(void*)&web_cnx,sizeof(web_cnx));
+			if(ret==-1)
+			{
+				perror("web connect");
+				goto eofunc;
+			}
+
+			{
+				GString *str;
+
+				str=g_string_new("");
+
+				g_string_sprintf(str,"CONNECT %s HTTP/1.1\nHost: %s\n",web_proxy_address,web_proxy_address);
+				/* proxy needs authentification ? */
+				if((socks_name!=NULL)&&(socks_pass!=NULL))
+				{
+					/* Yes */
+					int l1, l2;
+					char plain_pass[256];
+
+					l1=strlen(socks_name);
+					l2=strlen(socks_pass);
+					if((l1+l2+2)>sizeof(plain_pass))
+						fprintf(stderr,"ERROR: proxy login + password size exceeded buffer size, both are ignored\n");
+					else
+					{
+						char encoded_lp[512];
+
+						/* build the string "login:passwd" */
+						memcpy(plain_pass,socks_name,l1);
+						plain_pass[l1]=':';
+						memcpy(plain_pass+l1+1,socks_pass,l2);
+						plain_pass[l1+1+l2]='\0';
+
+						uu_auth(plain_pass,strlen(plain_pass),encoded_lp);
+
+						g_string_sprintfa(str,"Authorization: Basic %s\n",encoded_lp);
+					}
+				}
+				g_string_append_c(str,'\n');		/* end HTTP query */
+
+
+				if(send(desc,str->str,str->len,MSG_NOSIGNAL|MSG_DONTWAIT)!=str->len)
+				{
+					perror("web send CONNECT");
+					g_string_free(str,TRUE);
+					ret=-1;
+					goto eofunc;
+				}
+				g_string_free(str,TRUE);
+			}
+	
+			/* check if we receive a 200 code */
+			if(recv(desc,buf,4,MSG_WAITALL|MSG_NOSIGNAL)!=4)
+			{
+				ret=-1;
+				goto eofunc;
+			}
+
+			if(buf[0]!='2')
+			{
+				printf("connect thru web proxy returns %c%c%c.\n",buf[0],buf[1],buf[2]);
+				ret=-1;
+				goto eofunc;
+			}
+
+			/* now, we will discard any bytes until a '\n\n' was received */
+			{
+				gboolean connected=FALSE;
+				gboolean has_nl=FALSE;
+
+				while(!connected)
+				{
+					char c;
+					if(recv(desc,&c,1,MSG_WAITALL|MSG_NOSIGNAL)!=1)
+					{
+						perror("web receive fails");
+						ret=-1;
+						goto eofunc;
+					}
+
+					if(c=='\r')			/* ignore \r in the header */
+						continue;
+					if(c!='\n')
+					{
+						has_nl=FALSE;
+					}
+					else
+					{
+						if(!has_nl)
+							has_nl=TRUE;
+						else
+							connected=TRUE;
+					}
+				}
+			}
+			ret=0;
+			printf("web proxy connected\n");
+		}
 	}
 	else
 	{	/* SOCKS proxy enabled */
@@ -128,9 +305,6 @@ static int do_connect_opt_socks(int desc,struct sockaddr_in nom)
 				goto eofunc;
 			}
 	
-#if 0
-			printf("%X\n",val[1]);
-#endif
 			if(val[1]!=90)
 			{
 				fprintf(stderr,"socks CONNECT result: access denied or failed (%d)",(int)(val[1]));
@@ -510,11 +684,8 @@ int create_and_open_sock_on(char *hostname,unsigned short port, int non_block)
 	if(non_block)
 		set_non_bloquant_sock(s);
 
-#if 0
-	code=connect(s,(struct sockaddr*)&addr,sizeof addr);
-#else
+	/* perform the connect call which may use a proxy (web or SOCKS) */
 	code=do_connect_opt_socks(s,addr);
-#endif
 
 	if(non_block==0)
 	{
@@ -539,31 +710,76 @@ int create_and_open_sock_on(char *hostname,unsigned short port, int non_block)
 	return s;
 }
 
+/********************************************/
+/* check if the given value is an IP or not */
+/********************************************/
+/* output: TRUE=it is an IP, FALSE= no */
+/***************************************/
+static gboolean is_an_ip(const char *value_to_check)
+{
+	int a,b,c,d;
+
+	if(value_to_check==NULL)
+		return FALSE;
+
+	if(sscanf(value_to_check,"%d.%d.%d.%d",&a,&b,&c,&d)!=4)
+		return FALSE;		/* not enough value assigned */
+
+	if(
+		(a>=0)&&(a<=255)&&
+		(b>=0)&&(b<=255)&&
+		(c>=0)&&(c<=255)&&
+		(d>=0)&&(d<=255))
+	{
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 /******************************/
 /* return the default host IP */
 /************************************************************************/
 /* the default host IP is the IP of the interface used as default route */
 /************************************************************************/
-GString *get_default_host_ip(void)
+/* The parameter can be NULL and in this case, the default host IP is */
+/* used, it can also be an IP (IPv4) or an interface to have an IP    */
+/**********************************************************************/
+GString *get_default_host_ip(const char *ip_or_if, gboolean dynamic_flag)
 {
 	FILE *f;
 	char buf[512];
+	gboolean it_is_an_ip;
 
 	/* the following script does this: */
 	/* 1) extract the interface used by the default route: netstat -rn | grep ^0.0.0.0 | awk '{print $8;} */
 	/* 2) obtain the IP of this interface: netstat xxx | fgrep inet | cut -d : -f 2 | awk '{print $1;}' */
 	const char *cmd=
-#if 0
-		"/sbin/ifconfig `netstat -rn | grep ^0.0.0.0 | awk '{print $8;}'` | fgrep inet | cut -d : -f 2 | awk '{print $1;}'";
-#endif
-
 #ifndef ROUTE_HAS_GET_DEFAULT
 		"/sbin/ifconfig `netstat -rn | awk '/^0.0.0.0/{print$NF}'` | fgrep inet | cut -d : -f 2 | awk '{print $1;}'";
 #else
 		"/sbin/route -n get default | awk '/local/{print$NF}'";
 #endif
 
-	f=popen(cmd,"r");
+	it_is_an_ip=is_an_ip(ip_or_if);
+
+	/* if the parameter is an IP, we just return it but only in static mode */
+	if((it_is_an_ip==TRUE)&&(dynamic_flag==0))
+	{
+		return g_string_new(ip_or_if);
+	}
+
+	if((ip_or_if==NULL)||(strlen(ip_or_if)==0)||(it_is_an_ip==TRUE))
+	{
+		/* 3 possibles cases: no interface provided, empty string provided or IP provided */
+		strcpy(buf,cmd);
+	}
+	else
+	{
+		sprintf(buf,"/sbin/ifconfig '%s' | fgrep inet | cut -d : -f 2 | awk '{print $1;}'",ip_or_if);
+	}
+
+	f=popen(buf,"r");
 	if(f==NULL)
 	{
 		fprintf(stderr,"Unable to create a shell to obtain default IP.\n");
