@@ -1,7 +1,7 @@
 /* DCTC - a Direct Connect text clone for Linux
  * Copyright (C) 2001 Eric Prevoteau
  *
- * sema.c: Copyright (C) Eric Prevoteau <www@ac2i.tzo.com>
+ * sema.c: Copyright (C) Eric Prevoteau <www@a2pb.gotdns.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+/*
+$Id: sema.c,v 1.22 2003/12/28 08:12:38 uid68112 Exp $
+*/
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -25,6 +28,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <string.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <sys/types.h>
@@ -32,6 +36,7 @@
 #include <sys/sem.h>
 #include <limits.h>
 #include <pthread.h>
+#include <sched.h>
 #include <glib.h>
 
 #include "sema.h"
@@ -56,6 +61,9 @@
 
 #include "uaddr.h"
 #include "status.h"
+#include "var.h"
+#include "network.h"
+#include "gdl.h"
 
 /***********************************************************************/
 /* the following functions manage the bandwidth upload limit           */
@@ -208,6 +216,7 @@ int do_sema_init(char *keyfile, int *cur_semid, int spd_limit, int dl_spd_limit,
 	return 0;
 }
 
+#if 0
 /******************************************************************/
 /* this is the thread acting as clock                             */
 /* every second, it resets the 2nd semaphore to its initial value */
@@ -217,8 +226,6 @@ static void *sema_master(void *dm_val)
 	int semid=(int)dm_val;
 	FLAG1_STRUCT fs1;
 
-#ifndef BUGGY_LIBS
-
 	if(sizeof(fs1)!=sizeof(unsigned long int))
 		fprintf(stderr,"FLAG1_STRUCT has an invalid size (%d), result can be erroneous\n",sizeof(fs1));
 	/* set the is_clock_master flag of the gstatus */
@@ -229,29 +236,13 @@ static void *sema_master(void *dm_val)
 	while(1)
 	{
 		union semun v;
-#else
-	union semun v;								/* on some old buggy libs, calling semctl(GETVAL) from thread before calling SETVAL generate bus error */
-
-	if(sizeof(fs1)!=sizeof(unsigned long int))
-		fprintf(stderr,"FLAG1_STRUCT has an invalid size (%d), result can be erroneous\n",sizeof(fs1));
-	/* set the is_clock_master flag of the gstatus */
-	fs1.full=GET_GSTATUS_FLAG1();
-	fs1.bf.is_clock_master=1;
-	SET_GSTATUS_FLAG1(fs1.full);
-
-	v.val=0;										/* we use 0 to temporarily alter the speed without a significant impact */
-	if(semctl(semid,1,SETVAL,v)==-1)
-		perror("semctl0");
-	while(1)
-	{
-#endif
 		int i;
 
 		/* reset upload/download and gather speed */
 		for(i=0;i<3;i++)
 		{
 			/* reset speed */
-			v.val=semctl(semid,2+2*i,GETVAL);
+			v.val=semctl(semid,2+2*i,GETVAL,v);
 			if(v.val==-1)
 				v.val=SEMVMX;
 
@@ -268,8 +259,8 @@ static void *sema_master(void *dm_val)
 /**********************************************/
 static void create_sema_master(int semid)
 {
-	static pthread_attr_t thread_attr;	/* these 2 variables must exist as long as the thread exist */
-	static pthread_t thread_id;
+	static pthread_t thread_id; /* this variable must exist as long as the thread exist */
+	pthread_attr_t thread_attr;
 
    pthread_attr_init (&thread_attr);
    pthread_attr_setdetachstate(&thread_attr, PTHREAD_CREATE_DETACHED);
@@ -285,7 +276,9 @@ static void create_sema_master(int semid)
 		/* to reduce duplicated code, the DCTC being the clock master is also the one performing UADDR action */
 		create_uaddr_thread();
 	}
+	pthread_attr_destroy(&thread_attr);
 }
+#endif
 
 /******************************************************************************************/
 /* to avoid forever hanging of download, we must regularly check if a clock master exists */
@@ -296,8 +289,161 @@ void check_sema_master(int semid)
 
 	if(semop(semid,&sb,1)==0)
 	{
+		/* we have successfully obtain the semaphore */
+		/* 1) release it */
+		/* 2) start a new dctc_master process */
+		struct sembuf sb={0,+1,IPC_NOWAIT|SEM_UNDO};			/* master sema */
+		if(semop(semid,&sb,1)==0)
+		{
+			int p;
+			switch((p=fork()))
+			{
+				case -1:	/* fork fails */
+				case 0:	/* the child -> create dctc_master */
+							{
+								GPtrArray *array;
+								char buf[512];
+
+								array=g_ptr_array_new();
+								g_ptr_array_add(array,"dctc_master");
+								g_ptr_array_add(array,"-n");
+								g_ptr_array_add(array,org_nickname);
+								g_ptr_array_add(array,"-a");
+								g_ptr_array_add(array,host_ip);
+								sprintf(buf,"%u",com_port);
+								g_ptr_array_add(array,"-p");
+								g_ptr_array_add(array,strdup(buf));
+								g_ptr_array_add(array,"--precmd");
+								if(dl_on)
+									g_ptr_array_add(array,"/DLON");
+								else
+									g_ptr_array_add(array,"/DLOFF");
+								if(behind_fw)
+									g_ptr_array_add(array,"-f");
+								g_ptr_array_add(array,"--precmd");
+								if(when_done)
+									g_ptr_array_add(array,"/DONE");
+								else
+									g_ptr_array_add(array,"/UNDONE");
+								g_ptr_array_add(array,"--precmd");
+								if(with_ddl)
+									g_ptr_array_add(array,"/DDL");
+								else
+									g_ptr_array_add(array,"/NODDL");
+								g_ptr_array_add(array,"--precmd");
+								g_ptr_array_add(array,"/LINK");
+
+								g_ptr_array_add(array,"--precmd");
+								if(with_lazy_key_check)
+									g_ptr_array_add(array,"/LAZYKC");
+								else
+									g_ptr_array_add(array,"/NOLAZYKC");
+									
+								sprintf(buf,"/DFLAG with_sr_wake_up %d",with_sr_wake_up);
+								g_ptr_array_add(array,"--precmd");
+								g_ptr_array_add(array,strdup(buf));
+									
+								sprintf(buf,"/DFLAG min_gdl_wake_up_delay %d",min_gdl_wake_up_delay);
+								g_ptr_array_add(array,"--precmd");
+								g_ptr_array_add(array,strdup(buf));
+									
+								sprintf(buf,"/DFLAG dynamic_ip %d",dynamic_ip_flag);
+								g_ptr_array_add(array,"--precmd");
+								g_ptr_array_add(array,strdup(buf));
+									
+								if((org_host_ip!=NULL)&&(strlen(org_host_ip)))
+								{
+									sprintf(buf,"/IP %s",org_host_ip);
+									g_ptr_array_add(array,"--precmd");
+									g_ptr_array_add(array,strdup(buf));
+								}
+
+								sprintf(buf,"/DFLAG max_dl_per_user %d",max_dl_per_user);
+								g_ptr_array_add(array,"--precmd");
+								g_ptr_array_add(array,strdup(buf));
+									
+								sprintf(buf,"/DFLAG min_delay_between_search %d",min_delay_between_search);
+								g_ptr_array_add(array,"--precmd");
+								g_ptr_array_add(array,strdup(buf));
+									
+								sprintf(buf,"/DFLAG sharelist_dl %d",sharelist_dl_wo_slot);
+								g_ptr_array_add(array,"--precmd");
+								g_ptr_array_add(array,strdup(buf));
+								sprintf(buf,"/DFLAG fake_dcpp_client %d",fake_dcpp_client);
+								g_ptr_array_add(array,"--precmd");
+								g_ptr_array_add(array,strdup(buf));
+
+								if(fake_dcpp_version!=NULL)
+								{
+									sprintf(buf,"/DFLAG fake_dcpp_version %s",fake_dcpp_version->str);
+									g_ptr_array_add(array,"--precmd");
+									g_ptr_array_add(array,strdup(buf));
+								}
+									
+								sprintf(buf,"/DFLAG disp_user %d",disp_user);
+								g_ptr_array_add(array,"--precmd");
+								g_ptr_array_add(array,strdup(buf));
+
+								sprintf(buf,"/RECOND %u",recon_delay);
+								g_ptr_array_add(array,"--precmd");
+								g_ptr_array_add(array,strdup(buf));
+								sprintf(buf,"/MAXRUNGDLSRC %u",max_running_source_per_gdl);
+								g_ptr_array_add(array,"--precmd");
+								g_ptr_array_add(array,strdup(buf));
+								sprintf(buf,"/GDLASOFFAFT %u",disable_gdl_as_when_enough_running);
+								g_ptr_array_add(array,"--precmd");
+								g_ptr_array_add(array,strdup(buf));
+								sprintf(buf,"/UNODEPORT %u",unode_port);
+								g_ptr_array_add(array,"--precmd");
+								g_ptr_array_add(array,strdup(buf));
+								if(socks_ip!=NULL)
+								{
+									g_ptr_array_add(array,"-S");
+									g_ptr_array_add(array,socks_ip);
+								}
+								sprintf(buf,"%hu",socks_port);
+								g_ptr_array_add(array,"-P");
+								g_ptr_array_add(array,strdup(buf));
+								if(socks_name!=NULL)
+								{
+									g_ptr_array_add(array,"-X");
+									g_ptr_array_add(array,"-U");
+									g_ptr_array_add(array,socks_name);
+								}
+								
+								sprintf(buf,"/GDLASPORTS %u,%u",gdl_as_port_range[0],gdl_as_port_range[1]);
+								g_ptr_array_add(array,"--precmd");
+								g_ptr_array_add(array,strdup(buf));
+								
+								sprintf(buf,"/GDLMETPOLL %u",get_met_scan_interval());
+								g_ptr_array_add(array,"--precmd");
+								g_ptr_array_add(array,strdup(buf));
+								
+								{
+									const GString *gst=get_met_dir();
+
+									if((gst!=NULL)&&(gst->len!=0))
+									{
+										sprintf(buf,"/GDLMETDIR %s",gst->str);
+										g_ptr_array_add(array,"--precmd");
+										g_ptr_array_add(array,strdup(buf));
+									}
+								}
+								g_ptr_array_add(array,NULL);
+								execvp("dctc_master",(void*)(array->pdata));
+							}
+							perror("execlp fails");
+							_exit(1);
+				default:
+							fprintf(stderr,"New master created\n");
+							break;
+			}
+		}
+		else
+		{
+			perror("semop - release master sema");
+		}
 		/* to get slice, the function checks if the clock thread still runs */
-		create_sema_master(semid);
 	}
 }
 
@@ -328,6 +474,10 @@ void get_slice(int semid, SPD_SEMA semnum)
 /*******************************************/
 void get_ul_slices(int semid,int nb)
 {
+#ifdef _POSIX_PRIORITY_SCHEDULING
+	sched_yield();
+#endif
+
 	while(nb>0)
 	{
 		get_slice(semid,UL_SEMA);
@@ -342,6 +492,10 @@ void get_ul_slices(int semid,int nb)
 /*******************************************/
 void get_dl_slices(int semid,int nb)
 {
+#ifdef _POSIX_PRIORITY_SCHEDULING
+	sched_yield();
+#endif
+
 	while(nb>0)
 	{
 		get_slice(semid,DL_SEMA);
@@ -390,10 +544,11 @@ void release_ul_slot_controler(int semid)
 /********************************************************/
 void get_ul_slot_values(int semid,int *ttl_slot, int *busy_slot)
 {
+	union semun v;
 	lock_ul_slot_controler(semid);
 
-	*ttl_slot=semctl(semid,UL_SLOT_TTL_SEMA,GETVAL);
-	*busy_slot=semctl(semid,UL_SLOT_BUSY_SEMA,GETVAL);
+	*ttl_slot=semctl(semid,UL_SLOT_TTL_SEMA,GETVAL,v);
+	*busy_slot=semctl(semid,UL_SLOT_BUSY_SEMA,GETVAL,v);
 
 	release_ul_slot_controler(semid);
 }
@@ -405,11 +560,12 @@ void get_ul_slot_values(int semid,int *ttl_slot, int *busy_slot)
 /*******************************************************************/
 int number_of_free_slot(int semid)
 {
+	union semun v;
 	int ttl, busy;
 	int free_slt;
 
-	ttl=semctl(semid,UL_SLOT_TTL_SEMA,GETVAL);
-	busy=semctl(semid,UL_SLOT_BUSY_SEMA,GETVAL);
+	ttl=semctl(semid,UL_SLOT_TTL_SEMA,GETVAL,v);
+	busy=semctl(semid,UL_SLOT_BUSY_SEMA,GETVAL,v);
 
 	free_slt=ttl-busy;
 	if(free_slt<0)
@@ -428,26 +584,16 @@ static int nb_local_ul=0;			/* number of upload slots used by this client */
 /********************************************************/
 int try_to_get_ul_slot(int semid)
 {
+	union semun v;
 	int ttl_slot, busy_slot;
 	int ret=0;
 
 	lock_ul_slot_controler(semid);
 
-	ttl_slot=semctl(semid,UL_SLOT_TTL_SEMA,GETVAL);
-	busy_slot=semctl(semid,UL_SLOT_BUSY_SEMA,GETVAL);
+	ttl_slot=semctl(semid,UL_SLOT_TTL_SEMA,GETVAL,v);
+	busy_slot=semctl(semid,UL_SLOT_BUSY_SEMA,GETVAL,v);
 	if(busy_slot<ttl_slot)
 	{
-#if 0
-		union semun v;
-		ret=1;
-
-		v.val=busy_slot+1;
-		if(semctl(semid,UL_SLOT_BUSY_SEMA,SETVAL,v)==-1)
-		{
-			perror("try_to_get_ul_slot");
-			ret=0;
-		}
-#else
 		/* we must use this command because the previous code */
 		/* does not free the ul slot if the program ends */
 		struct sembuf get_one_ul_slot={UL_SLOT_BUSY_SEMA,+1,SEM_UNDO};
@@ -458,7 +604,6 @@ int try_to_get_ul_slot(int semid)
 			perror("try_to_get_ul_slot");
 			ret=0;
 		}
-#endif
 	}
 
 	if(ret)
@@ -480,17 +625,6 @@ int try_to_get_ul_slot(int semid)
 /********************************************************/
 void force_get_ul_slot(int semid)
 {
-#if 0
-	int busy_slot;
-	union semun v;
-	lock_ul_slot_controler(semid);
-	busy_slot=semctl(semid,UL_SLOT_BUSY_SEMA,GETVAL);
-	v.val=busy_slot+1;
-	if(semctl(semid,UL_SLOT_BUSY_SEMA,SETVAL,v)==-1)
-	{
-		perror("force_get_ul_slot");
-	}
-#else
 	struct sembuf get_one_ul_slot={UL_SLOT_BUSY_SEMA,+1,SEM_UNDO};
 
 	lock_ul_slot_controler(semid);
@@ -498,7 +632,6 @@ void force_get_ul_slot(int semid)
 	{
 		perror("force_get_ul_slot");
 	}
-#endif
 	nb_local_ul++;
 
 	release_ul_slot_controler(semid);
@@ -513,25 +646,17 @@ void force_get_ul_slot(int semid)
 /*******************************************************/
 void free_one_ul_slot(int semid)
 {
+	union semun v;
 	int busy_slot;
 	lock_ul_slot_controler(semid);
-	busy_slot=semctl(semid,UL_SLOT_BUSY_SEMA,GETVAL);
+	busy_slot=semctl(semid,UL_SLOT_BUSY_SEMA,GETVAL,v);
 	if(busy_slot>0)
 	{
-#if 0
-		union semun v;
-		v.val=busy_slot-1;
-		if(semctl(semid,UL_SLOT_BUSY_SEMA,SETVAL,v)==-1)
-		{
-			perror("free_one_ul_slot");
-		}
-#else
 		struct sembuf free_one_ul_slot_op={UL_SLOT_BUSY_SEMA,-1,SEM_UNDO};
 		if(semop(semid,&free_one_ul_slot_op,1)==-1)
 		{
 			perror("free_one_ul_slot");
 		}
-#endif
 		nb_local_ul--;
 	}
 	else
@@ -565,10 +690,11 @@ void set_number_of_ul_slot(int semid, unsigned int number_of_slot)
 /*********************************/
 int get_number_of_ul_slot(int semid)
 {
+	union semun v;
 	int ttl_slot;
 
 	lock_ul_slot_controler(semid);
-	ttl_slot=semctl(semid,UL_SLOT_TTL_SEMA,GETVAL);
+	ttl_slot=semctl(semid,UL_SLOT_TTL_SEMA,GETVAL,v);
 	release_ul_slot_controler(semid);
 
 	return ttl_slot;
