@@ -1,7 +1,7 @@
 /* DCTC - a Direct Connect text clone for Linux
  * Copyright (C) 2001 Eric Prevoteau
  *
- * typical_action.c: Copyright (C) Eric Prevoteau <www@ac2i.tzo.com>
+ * typical_action.c: Copyright (C) Eric Prevoteau <www@a2pb.gotdns.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+/*
+$Id: typical_action.c,v 1.8 2003/12/28 08:12:38 uid68112 Exp $
+*/
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -30,6 +33,7 @@
 #include "typical_action.h"
 #include "dc_com.h"
 #include "var.h"
+#include "sema.h"
 
 static GString *last_sent_myinfo=NULL;
 
@@ -43,18 +47,58 @@ static GString *last_sent_myinfo=NULL;
 /*			bbb is e-mail (empty string)                                */
 /*			yyy is size of shared data in bytes                         */
 /*********************************************************************/
+/* Note: the function may automatically add a TAG to the MyINFO */
+/****************************************************************/
 void set_user_info(int sck,char *nick, char *description, char *cnx,char cnx_opt,char *mail,double amount)
 {
 	GString *user_info;
 
 	user_info=g_string_new("$MyINFO $ALL ");
-	g_string_sprintfa(user_info,"%s %s$ $%s%c$%s$%.0lf$",
+	if((description_tag_auto==FALSE)&&(fake_dcpp_client==FALSE))
+	{
+		g_string_sprintfa(user_info,"%s %s$ $%s%c$%s$%.0lf$",
 										nick,
 										(description!=NULL)?description:"",
 										cnx,
 										cnx_opt,
 										(mail!=NULL)?mail:"",
 										amount+offset_sizeof_data);
+	}
+	else
+	{
+		int ttl_dl_slot;
+		int busy_slot;
+
+		get_ul_slot_values(bl_semid,&ttl_dl_slot,&busy_slot);
+
+		if((fake_dcpp_client==FALSE)||(fake_dcpp_version==NULL)||(fake_dcpp_version->len==0))
+		{
+			g_string_sprintfa(user_info,"%s %s <DCTC V:" VERSION ",M:%c,H:%d,S:%d>$ $%s%c$%s$%.0lf$",
+											nick,
+											(description!=NULL)?description:"",
+											(behind_fw?'P':'A'),		/* automatically assign the letter according to the current FW mode */
+											num_running_dctc(),		/* there is no way to know which DCTC runs with a registered user, report all running clients */
+											ttl_dl_slot,
+											cnx,
+											cnx_opt,
+											(mail!=NULL)?mail:"",
+											amount+offset_sizeof_data);
+		}
+		else
+		{
+			g_string_sprintfa(user_info,"%s %s <++ V:%s,M:%c,H:%d/0/0,S:%d>$ $%s%c$%s$%.0lf$",
+											nick,
+											(description!=NULL)?description:"",
+											fake_dcpp_version->str,
+											(behind_fw?'P':'A'),		/* automatically assign the letter according to the current FW mode */
+											num_running_dctc(),		/* there is no way to know which DCTC runs with a registered user, report all running clients */
+											ttl_dl_slot,
+											cnx,
+											cnx_opt,
+											(mail!=NULL)?mail:"",
+											amount+offset_sizeof_data);
+		}
+	}
 
 	if(last_sent_myinfo!=NULL)
 	{	/* a string already exist */
