@@ -1,3 +1,26 @@
+/* DCTC - a Direct Connect text clone for Linux
+ * Copyright (C) 2001 Eric Prevoteau
+ *
+ * uaddr.c: Copyright (C) Eric Prevoteau <www@a2pb.gotdns.org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
+/*
+$Id: uaddr.c,v 1.8 2003/12/28 08:12:38 uid68112 Exp $
+*/
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -204,8 +227,8 @@ static void *uaddr_thread(void *dummy)
 /******************************************************************************/
 void create_uaddr_thread(void)
 {
-	static pthread_attr_t thread_attr;		/* these 2 variables must exists as long as the thread exists */
-	static pthread_t thread_id;
+	static pthread_t thread_id; /* this variable must exist as long as the thread exists */
+	pthread_attr_t thread_attr;
 
 	pthread_attr_init (&thread_attr);
 	pthread_attr_setdetachstate(&thread_attr, PTHREAD_CREATE_DETACHED);
@@ -213,6 +236,7 @@ void create_uaddr_thread(void)
 	{
 		disp_msg(ERR_MSG,"create_uaddr_thread","Fatal Error","Fail to start UADDR thread",NULL);
 	}
+	pthread_attr_destroy(&thread_attr);
 }
 
 /********************************/
@@ -228,7 +252,7 @@ void init_uaddr(void)
 	tmp=g_string_new(dctc_dir->str);
 	g_string_sprintfa(tmp,"/uaddr_ready.%d",sizeof(UADDR_READY));
 
-	uaddr_ready=lmp_new(tmp->str,sizeof(UADDR_READY));
+	uaddr_ready=lmp_new(tmp->str,sizeof(UADDR_READY),-1);
 	if(uaddr_ready==NULL)
 	{
 		disp_msg(ERR_MSG,"init_uaddr","unable to create ","|s",tmp->str,NULL);
@@ -240,7 +264,7 @@ void init_uaddr(void)
 	tmp=g_string_assign(tmp,dctc_dir->str);
 	g_string_sprintfa(tmp,"/uaddr_partial.%d",sizeof(UADDR_PARTIAL));
 
-	uaddr_partial=lmp_new(tmp->str,sizeof(UADDR_PARTIAL));
+	uaddr_partial=lmp_new(tmp->str,sizeof(UADDR_PARTIAL),-1);
 	if(uaddr_partial==NULL)
 	{
 		lmp_close(uaddr_ready);
@@ -320,6 +344,75 @@ void delete_uaddr_entry_by_addr(char *dl_addr)
 	G_UNLOCK(uaddr_partial);
 }
 
+/*****************************************************/
+/* check if the given address exists in the database */
+/*****************************************************/
+/* output: 1=yes, 0=no */
+/***********************/
+int check_uaddr_entry_by_addr(char *dl_addr)
+{
+	int i;
+	UADDR_READY *uar;
+	int ret=0;
+
+	G_LOCK(uaddr_ready);
+
+	if(lmp_lock_and_map(uaddr_ready))
+	{
+		G_UNLOCK(uaddr_ready);
+		return ret;
+	}
+
+	uar=uaddr_ready->mapped_addr;
+	for(i=1;i<uaddr_ready->nb_records;i++)	/* skip the first record */
+	{
+		if(uar[i].valid)
+		{
+			if(!strcmp(uar[i].dl_addr,dl_addr))
+			{
+				if((uar[i].cyclic_dirty==0)||(uar[i].cyclic_dirty==1))	/* currently valid ? */
+					ret=1;	/* found */
+				break;
+			}
+		}
+	}
+
+	lmp_unmap_and_unlock(uaddr_ready);
+	G_UNLOCK(uaddr_ready);
+
+	if(!ret)
+	{
+		/* not yet found, check the partial db */
+		UADDR_PARTIAL *uap;
+
+		G_LOCK(uaddr_partial);
+
+		if(lmp_lock_and_map(uaddr_partial))
+		{
+			G_UNLOCK(uaddr_partial);
+			return ret;
+		}
+
+		uap=uaddr_partial->mapped_addr;
+		for(i=1;i<uaddr_partial->nb_records;i++)	/* skip the first record */
+		{
+			if(uap[i].valid)
+			{
+				if(!strcmp(uap[i].dl_addr,dl_addr))
+				{
+					ret=1;
+					break;
+				}
+			}
+		}
+
+		lmp_unmap_and_unlock(uaddr_partial);
+		G_UNLOCK(uaddr_partial);
+	}	
+
+	return ret;
+}
+
 /**********************************************************/
 /* check if the given nickname has a known remote address */
 /**********************************************************/
@@ -544,6 +637,146 @@ int add_uaddr_entry_addr_dl_only(char *dl_addr)
 	return 0;
 }
 
+/**********************************************************/
+/* the following function adds an entry to the UADDR file */
+/**********************************************************/
+/* addr_dl is always uniq in the database */
+/******************************************/
+/* output: 0= ok    */
+/*         1= error */
+/****************************************************/
+/* the function is smarter than the previous one    */
+/* it also checks if the address does not exist in  */
+/* the uaddr_ready DB and if it does not exist with */
+/* a different port.                                */
+/****************************************************/
+int add_uaddr_entry_addr_dl_only_xtra(char *dl_addr)
+{
+	int i;
+	int pos=-1;
+	UADDR_PARTIAL *uap;
+	int empty=-1;
+	char *t;
+	int addr_len;
+	int added=-1;
+
+	t=strchr(dl_addr,':');
+	if(t==NULL)
+		return 1;
+	t++;		/* keep the ':' at the end */
+	addr_len=t-dl_addr;
+
+	G_LOCK(uaddr_partial);
+
+	if(lmp_lock_and_map(uaddr_partial))
+	{
+		G_UNLOCK(uaddr_partial);
+		return 1;
+	}
+
+	uap=uaddr_partial->mapped_addr;
+	for(i=1;i<uaddr_partial->nb_records;i++)	/* skip the first record */
+	{
+		if(uap[i].valid)
+		{
+			if(!strncmp(uap[i].dl_addr,dl_addr,addr_len))
+			{
+				pos=i;
+				break;
+			}
+		}
+		else
+		{
+			if(empty==-1)
+				empty=i;
+		}
+	}
+
+	if(pos!=-1)
+	{
+		uap[pos].ping_counter=0;
+	}
+	else
+	{
+		int ret=-1;
+
+		/* the entry does not exist but perhaps it is already in the uaddr_ready table */
+		UADDR_READY *uar;
+
+		G_LOCK(uaddr_ready);
+
+		if(!lmp_lock_and_map(uaddr_ready))
+		{
+			uar=uaddr_ready->mapped_addr;
+			for(i=1;i<uaddr_ready->nb_records;i++)	/* skip the first record */
+			{
+				if(uar[i].valid)
+				{
+					if(!strncmp(uar[i].dl_addr,dl_addr,addr_len))
+					{
+						ret=i;	/* found */
+						break;
+					}
+				}
+			}
+			lmp_unmap_and_unlock(uaddr_ready);
+		}
+		G_UNLOCK(uaddr_ready);
+
+		if(ret==-1)
+		{
+			added=1;
+
+			/* not exist anywhere */
+			if(empty!=-1)
+			{
+				uap[empty].valid=1;
+				strncpy_max(uap[empty].dl_addr,dl_addr,UADDR_DL);
+				uap[empty].ping_counter=0;
+			}
+			else
+			{
+				UADDR_PARTIAL nw;
+	
+				nw.valid=1;
+				strncpy_max(nw.dl_addr,dl_addr,UADDR_DL);
+				nw.ping_counter=0;
+	
+				lmp_append_record(uaddr_partial,&nw);
+			}
+		}
+	}
+	lmp_unmap_and_unlock(uaddr_partial);
+	G_UNLOCK(uaddr_partial);
+	if(added)
+	{
+		GString *ndl;
+		int port_num;
+
+		ndl=g_string_new(dl_addr);
+
+		port_num=atoi(dl_addr+addr_len);		/* current port added */
+
+		if(port_num!=412)
+		{
+			g_string_truncate(ndl,addr_len);
+			g_string_append(ndl,"412");
+			add_uaddr_entry_addr_dl_only(ndl->str);
+		}
+
+		if(port_num!=1412)
+		{
+			g_string_truncate(ndl,addr_len);
+			g_string_append(ndl,"1412");
+			add_uaddr_entry_addr_dl_only(ndl->str);
+		}
+
+		g_string_free(ndl,TRUE);
+	}
+	return 0;
+}
+
+
 #if 0
 /*************************************************************************************************/
 /* the following function increases the ping_counter of the UADDR entry having the given address */
@@ -770,7 +1003,7 @@ static void	init_uaddr_node(void)
 		tmp=g_string_new(dctc_dir->str);
 		g_string_sprintfa(tmp,"/unode.%d",sizeof(UNODE_DATA));
 
-		unode_lmp=lmp_new(tmp->str,sizeof(UNODE_DATA));
+		unode_lmp=lmp_new(tmp->str,sizeof(UNODE_DATA),-1);
 		if(unode_lmp==NULL)
 		{
 			disp_msg(ERR_MSG,"init_uaddr_node","unable to create ","|s",tmp->str,NULL);
@@ -1677,27 +1910,6 @@ static void manage_uaddr_node(void)
 			}
 		}
 	}
-
-#if 0
-	if(incoming_sock!=NULL)
-	{
-		printf("incoming: %d\n",incoming_sock->len);
-	}
-	if(outgoing_sock!=NULL)
-	{
-		int i;
-		printf("outgoing: %d\n",outgoing_sock->len);
-
-		for(i=0;i<outgoing_sock->len;i++)
-		{
-			DCTC2DCTC_CNX *d2dcnx;
-
-			d2dcnx=&(g_array_index(outgoing_sock,DCTC2DCTC_CNX,i));
-
-			printf("  %d:%d = %s\n",i,d2dcnx->sock_fd,d2dcnx->ip_addr);
-		}
-	}
-#endif
 }
 
 
