@@ -1,7 +1,7 @@
 /* DCTC - a Direct Connect text clone for Linux
  * Copyright (C) 2001 Eric Prevoteau
  *
- * var.h: Copyright (C) Eric Prevoteau <www@ac2i.tzo.com>
+ * var.h: Copyright (C) Eric Prevoteau <www@a2pb.gotdns.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+/*
+$Id: var.h,v 1.13 2004/01/09 17:15:43 ericprev Exp $
+*/
 
 #ifndef __VAR_H__
 #define __VAR_H__
@@ -57,15 +60,6 @@ extern double sizeof_data;          /* size of the data shared by this client */
 extern double offset_sizeof_data;	/* when the client has to return the size of data it shares, this value */
 												/* is added to the previous one */
 
-#if 0
-extern int free_dl_slot;         /* number of free download slot */
-                                 /* NOTE: this number can be negative for 2 reasons: */
-                                 /* 1) you have reduced the ttl_dl_slot value under the number of running uploads */
-                                 /* 2) a user having the IGNORE_SLOT_LIMIT flag is downloading */
-
-extern int ttl_dl_slot;             /* number of existing download slot */
-#endif
-
 extern int dl_on;                /* this flag enables or disables the upload capability */
                                  /* if ==1, other users can download from you */
                                  /* if ==0, other users cannot download from you */
@@ -78,6 +72,8 @@ extern char *nick_passwd;			/* when the hub wants a password. We will use the on
 
 extern GString *dctc_rating_gs;	/* this value is not used by DCTC, it is only kept here */
 
+extern int description_tag_auto;	/* if set, the client will automatically attached tag to the user description */
+
 /**************************************/
 /* description of the connection type */
 /**************************************/
@@ -89,6 +85,9 @@ extern unsigned short com_port;     /* port available, to receive data if not be
                                     /* because port below 1024 are not allowed for      */
                                     /* non-privileged users under Un*x                  */
 extern char *host_ip;               /* ip of this host */
+extern char *org_host_ip;				/* this is where the command /IP put its content */
+												/* because the command can provided a FQDN or even a network interface */
+												/* we cannot load directly host_ip with it */
 
 extern int dynamic_ip_flag;      /* if ==0, once host_ip has been filled, it is never updated (static IP) */
                                  /* else ==1, the host_ip is recomputed each time before connecting the hub */
@@ -99,10 +98,10 @@ extern GString *hubip;       /* IP of the hub where we are connected */
 extern unsigned int hub_port; /* port of the hub where we are connected (411 is the default port) */
 extern GString *hubname;     /* Name of the hub where we are connected (filed when connected) */
 
-extern unsigned int hub_tos; // TOS value to set on hub connections
-extern unsigned int udp_tos; // TOS value to set on udp (search) pakets
-extern unsigned int dl_tos; // TOS value to set on download connections
-extern unsigned int ul_tos; // TOS value to set on upload connections
+extern unsigned int hub_tos; /* TOS value to set on hub connections */
+extern unsigned int udp_tos; /* TOS value to set on udp (search) pakets */
+extern unsigned int dl_tos; /* TOS value to set on download connections */
+extern unsigned int ul_tos; /* TOS value to set on upload connections */
 
 extern unsigned int unode_port;   /* default port number for UNODE */
 extern unsigned int wanted_unode_port;  /* port number for UNODE. If you want to change UNODE port, change this value, not the one above */
@@ -127,20 +126,9 @@ extern int keyb_fd;        /* it is the keyboard fd */
 extern int when_done;      /* when a download is done, ==0, do nothing */
                            /* ==1, move the file into the done/ directory */
 
-extern int wanna_emode;		/* when the --mdc flag is used, this variable contains the flag value */
-extern int having_emode;	/* depending of the connected hub, the current emode may differ from the */
-                           /* wanted emode, 0 is the default DC mode */
-
-extern int with_md5sum;		/* enable/disable md5sum for content search */
-
 extern int with_ddl;			/* enable/disable dDL capability */
-extern int with_dctclink;	/* enable/disable link between dctc clients */
 extern int grab_ban_ip;		/* enable/disable IP grabbing of banned users */
 
-extern int with_dl_force;	/* enable/disable download forcing when a twin download occurs */
-
-extern int hide_absolute;	/* hide/show the / (\ seen on DC) at the beginning of the returned search result and file list */
-
 extern int abort_upload_when_user_leaves;
                            /* abort the remote user upload when a user leaves the hub */
 
@@ -164,6 +152,15 @@ extern int gdl_as_port_range[2];     /* lower and upper value of GDL autoscan po
 
 extern int min_delay_between_search;	/* delay to wait between 2 search queries */
 
+extern unsigned int sharelist_dl_wo_slot;   /* allow downloading of your sharelist by clients even if no slots are available */
+
+extern unsigned int fake_dcpp_client;			/* simulate DC++ tag and key */
+extern GString *fake_dcpp_version;			/* version of DC++ to simulate */
+
+extern unsigned int disp_user;              /* disable/enable USER+,USER-, ... and anything related to user change message (1=enable, 0=disable)*/
+
+extern GPtrArray *client_capabilities;   /* list of capabilities of the client */
+
 /************************************************************/
 /* debug mode. ==0, no debug. !=0, debug mode               */
 /* if debug_mode is set, all debug messages will be printed */
